//
// ApacheServerRequest.h
//
// $Id: //poco/1.4/ApacheConnector/include/ApacheServerRequest.h#2 $
//
// Copyright (c) 2006-2011, Applied Informatics Software Engineering GmbH.
// and Contributors.
//
// Permission is hereby granted, free of charge, to any person or organization
// obtaining a copy of the software and accompanying documentation covered by
// this license (the "Software") to use, reproduce, display, distribute,
// execute, and transmit the Software, and to prepare derivative works of the
// Software, and to permit third-parties to whom the Software is furnished to
// do so, all subject to the following:
// 
// The copyright notices in the Software and this entire statement, including
// the above license grant, this restriction and the following disclaimer,
// must be included in all copies of the Software, in whole or in part, and
// all derivative works of the Software, unless such copies or derivative
// works are solely in the form of machine-executable object code generated by
// a source language processor.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
// SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
// FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.
//


#ifndef ApacheConnector_ApacheServerRequest_INCLUDED
#define ApacheConnector_ApacheServerRequest_INCLUDED


#include "ApacheConnector.h"
#include "ApacheStream.h"
#include "Poco/Net/HTTPServerRequest.h"
#include <set>


class ApacheServerResponse;


class ApacheServerRequest: public Poco::Net::HTTPServerRequest
{
public:
	ApacheServerRequest(
		ApacheRequestRec* pApacheRequest, 
		const char* serverName, 
		int serverPort, 
		const char* clientName, 
		int clientPort);
		/// Creates a new ApacheServerRequest.

	~ApacheServerRequest();
		/// Destroys the ApacheServerRequest.

	std::istream& stream();
		/// Returns the input stream for reading
		/// the request body.
		///
		/// The stream is valid until the HTTPServerRequest
		/// object is destroyed.

	bool expectContinue() const;
		/// Returns true if the client expects a
		/// 100 Continue response.

	const Poco::Net::SocketAddress& clientAddress() const;
		/// Returns the client's address.

	const Poco::Net::SocketAddress& serverAddress() const;
		/// Returns the server's address.

	const Poco::Net::HTTPServerParams& serverParams() const;
		/// Returns a reference to the server parameters.

	Poco::Net::HTTPServerResponse& response() const;
		/// Returns a reference to the associated response

protected:
	void setResponse(ApacheServerResponse* pResponse);

private:
	ApacheRequestRec*        _pApacheRequest;
	ApacheServerResponse*    _pResponse;
	ApacheInputStream*       _pStream;
	Poco::Net::SocketAddress _serverAddress;
	Poco::Net::SocketAddress _clientAddress;
	
	friend class ApacheServerResponse;
};


//
// inlines
//
inline std::istream& ApacheServerRequest::stream()
{
	poco_check_ptr (_pStream);
	
	return *_pStream;
}


inline const Poco::Net::SocketAddress& ApacheServerRequest::clientAddress() const
{
	return _clientAddress;
}


inline const Poco::Net::SocketAddress& ApacheServerRequest::serverAddress() const
{
	return _serverAddress;
}


#endif // ApacheConnector_ApacheServerRequest_INCLUDED
