//
// FIFOBufferStream.cpp
//
// $Id: //poco/1.4/Foundation/src/FIFOBufferStream.cpp#1 $
//
// Library: Foundation
// Package: Streams
// Module:  FIFOBufferStream
//
// Copyright (c) 2004-2006, Applied Informatics Software Engineering GmbH.
// and Contributors.
//
// Permission is hereby granted, free of charge, to any person or organization
// obtaining a copy of the software and accompanying documentation covered by
// this license (the "Software") to use, reproduce, display, distribute,
// execute, and transmit the Software, and to prepare derivative works of the
// Software, and to permit third-parties to whom the Software is furnished to
// do so, all subject to the following:
// 
// The copyright notices in the Software and this entire statement, including
// the above license grant, this restriction and the following disclaimer,
// must be included in all copies of the Software, in whole or in part, and
// all derivative works of the Software, unless such copies or derivative
// works are solely in the form of machine-executable object code generated by
// a source language processor.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
// SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
// FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.
//


#include "Poco/FIFOBufferStream.h"


namespace Poco {


//
// FIFOBufferStreamBuf
//


FIFOBufferStreamBuf::FIFOBufferStreamBuf():
	BufferedBidirectionalStreamBuf(STREAM_BUFFER_SIZE + 4, std::ios::in | std::ios::out),
	_pFIFOBuffer(new FIFOBuffer(STREAM_BUFFER_SIZE, true)),
	_fifoBuffer(*_pFIFOBuffer)
{
}


FIFOBufferStreamBuf::FIFOBufferStreamBuf(FIFOBuffer& fifoBuffer):
	BufferedBidirectionalStreamBuf(fifoBuffer.size() + 4, std::ios::in | std::ios::out),
	_pFIFOBuffer(0),
	_fifoBuffer(fifoBuffer)
{
	fifoBuffer.setNotify(true);
}


FIFOBufferStreamBuf::FIFOBufferStreamBuf(char* pBuffer, std::size_t length):
	BufferedBidirectionalStreamBuf(length + 4, std::ios::in | std::ios::out),
	_pFIFOBuffer(new FIFOBuffer(pBuffer, length, true)),
	_fifoBuffer(*_pFIFOBuffer)
{
}


FIFOBufferStreamBuf::FIFOBufferStreamBuf(const char* pBuffer, std::size_t length):
	BufferedBidirectionalStreamBuf(length + 4, std::ios::in | std::ios::out),
	_pFIFOBuffer(new FIFOBuffer(pBuffer, length, true)),
	_fifoBuffer(*_pFIFOBuffer)
{
}


FIFOBufferStreamBuf::FIFOBufferStreamBuf(std::size_t length):
	BufferedBidirectionalStreamBuf(length + 4, std::ios::in | std::ios::out),
	_pFIFOBuffer(new FIFOBuffer(length, true)),
	_fifoBuffer(*_pFIFOBuffer)
{
}


FIFOBufferStreamBuf::~FIFOBufferStreamBuf()
{
	delete _pFIFOBuffer;
}


int FIFOBufferStreamBuf::readFromDevice(char* buffer, std::streamsize length)
{
	poco_assert (length > 0);
	return _fifoBuffer.read(buffer, static_cast<std::size_t>(length));
}


int FIFOBufferStreamBuf::writeToDevice(const char* buffer, std::streamsize length)
{
	poco_assert (length > 0);
	return _fifoBuffer.write(buffer, static_cast<std::size_t>(length));
}


//
// FIFOIOS
//


FIFOIOS::FIFOIOS(FIFOBuffer& fifoBuffer): _buf(fifoBuffer)
{
	poco_ios_init(&_buf);
}


FIFOIOS::FIFOIOS(char* pBuffer, std::size_t length): _buf(pBuffer, length)
{
	poco_ios_init(&_buf);
}


FIFOIOS::FIFOIOS(const char* pBuffer, std::size_t length): _buf(pBuffer, length)
{
	poco_ios_init(&_buf);
}


FIFOIOS::FIFOIOS(std::size_t length): _buf(length)
{
	poco_ios_init(&_buf);
}


FIFOIOS::~FIFOIOS()
{
	try
	{
		_buf.sync();
	}
	catch (...)
	{
	}
}


FIFOBufferStreamBuf* FIFOIOS::rdbuf()
{
	return &_buf;
}


void FIFOIOS::close()
{
	_buf.sync();
}


//
// FIFOBufferStream
//


FIFOBufferStream::FIFOBufferStream(FIFOBuffer& fifoBuffer):
	FIFOIOS(fifoBuffer),
    std::iostream(&_buf),
	readable(_buf.fifoBuffer().readable),
	writable(_buf.fifoBuffer().writable)
{
}


FIFOBufferStream::FIFOBufferStream(char* pBuffer, std::size_t length):
	FIFOIOS(pBuffer, length),
    std::iostream(&_buf),
	readable(_buf.fifoBuffer().readable),
	writable(_buf.fifoBuffer().writable)
{
}


FIFOBufferStream::FIFOBufferStream(const char* pBuffer, std::size_t length):
	FIFOIOS(pBuffer, length),
    std::iostream(&_buf),
	readable(_buf.fifoBuffer().readable),
	writable(_buf.fifoBuffer().writable)
{
}


FIFOBufferStream::FIFOBufferStream(std::size_t length):
	FIFOIOS(length),
    std::iostream(&_buf),
	readable(_buf.fifoBuffer().readable),
	writable(_buf.fifoBuffer().writable)
{
}


FIFOBufferStream::~FIFOBufferStream()
{
}


} // namespace Poco
