//
// FIFOBufferStreamTest.cpp
//
// $Id: //poco/1.4/Foundation/testsuite/src/FIFOBufferStreamTest.cpp#2 $
//
// Copyright (c) 2004-2006, Applied Informatics Software Engineering GmbH.
// and Contributors.
//
// Permission is hereby granted, free of charge, to any person or organization
// obtaining a copy of the software and accompanying documentation covered by
// this license (the "Software") to use, reproduce, display, distribute,
// execute, and transmit the Software, and to prepare derivative works of the
// Software, and to permit third-parties to whom the Software is furnished to
// do so, all subject to the following:
// 
// The copyright notices in the Software and this entire statement, including
// the above license grant, this restriction and the following disclaimer,
// must be included in all copies of the Software, in whole or in part, and
// all derivative works of the Software, unless such copies or derivative
// works are solely in the form of machine-executable object code generated by
// a source language processor.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
// SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
// FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.
//


#include "FIFOBufferStreamTest.h"
#include "CppUnit/TestCaller.h"
#include "CppUnit/TestSuite.h"
#include "Poco/FIFOBuffer.h"
#include "Poco/FIFOBufferStream.h"
#include "Poco/Delegate.h"


using Poco::FIFOBuffer;
using Poco::FIFOBufferStream;
using Poco::delegate;


FIFOBufferStreamTest::FIFOBufferStreamTest(const std::string& name): CppUnit::TestCase(name)
{
}


FIFOBufferStreamTest::~FIFOBufferStreamTest()
{
}


void FIFOBufferStreamTest::testInput()
{
	const char* data = "This is a test";
	FIFOBuffer fb1(data, 14);
	FIFOBufferStream str1(fb1);
	assert (str1.rdbuf()->fifoBuffer().isFull());

	int c = str1.get();
	assert (c == 'T');
	c = str1.get();
	assert (c == 'h');
	
	std::string str;
	str1 >> str;
	assert (str == "is");
	
	char buffer[32];
	str1.read(buffer, sizeof(buffer));
	assert (str1.gcount() == 10);
	buffer[str1.gcount()] = 0;
	assert (std::string(" is a test") == buffer);
	
	const char* data2 = "123";
	FIFOBufferStream str2(data2, 3);

	c = str2.get();
	assert (c == '1');
	assert (str2.good());
	c = str2.get();
	assert (c == '2');
	str2.unget();
	c = str2.get();
	assert (c == '2');
	assert (str2.good());
	c = str2.get();
	assert (c == '3');
	assert (str2.good());
	c = str2.get();
	assert (c == -1);
	assert (str2.eof());
}


void FIFOBufferStreamTest::testOutput()
{
	char output[64];
	FIFOBufferStream iostr1(output, 64);
	iostr1 << "This is a test " << 42 << std::ends << std::flush;
	assert (std::string("This is a test 42") == output);
}


void FIFOBufferStreamTest::testNotify()
{
	FIFOBuffer fb(18);
	FIFOBufferStream iostr(fb);
	assert (iostr.rdbuf()->fifoBuffer().isEmpty());

	assert (0 == _readableToNot);
	assert (0 == _notToReadable);
	assert (0 == _writableToNot);
	assert (0 == _notToWritable);

	iostr.readable += delegate(this, &FIFOBufferStreamTest::onReadable);
	iostr.writable += delegate(this, &FIFOBufferStreamTest::onWritable);

	iostr << "This is a test " << 42 << std::ends << std::flush;
	assert (iostr.rdbuf()->fifoBuffer().isFull());

	assert (0 == _readableToNot);
	assert (1 == _notToReadable);
	assert (1 == _writableToNot);
	assert (0 == _notToWritable);

	char input[64];
	iostr >> input;
	assert (std::string("This") == input);
	assert (iostr.rdbuf()->fifoBuffer().isEmpty());
	
	assert (1 == _readableToNot);
	assert (1 == _notToReadable);
	assert (1 == _writableToNot);
	assert (1 == _notToWritable);

	iostr >> input;
	assert (std::string("is") == input);

	assert (1 == _readableToNot);
	assert (1 == _notToReadable);
	assert (1 == _writableToNot);
	assert (1 == _notToWritable);

	iostr >> input;
	assert (std::string("a") == input);

	assert (1 == _readableToNot);
	assert (1 == _notToReadable);
	assert (1 == _writableToNot);
	assert (1 == _notToWritable);

	iostr >> input;
	assert (std::string("test") == input);

	assert (1 == _readableToNot);
	assert (1 == _notToReadable);
	assert (1 == _writableToNot);
	assert (1 == _notToWritable);

	iostr >> input;
	assert (std::string("42") == input);

	assert (1 == _readableToNot);
	assert (1 == _notToReadable);
	assert (1 == _writableToNot);
	assert (1 == _notToWritable);

	iostr.clear();
	assert (iostr.good());
	iostr << "This is a test " << 42 << std::ends << std::flush;
	assert (iostr.rdbuf()->fifoBuffer().isFull());

	assert (1 == _readableToNot);
	assert (2 == _notToReadable);
	assert (2 == _writableToNot);
	assert (1 == _notToWritable);

	iostr.readable -= delegate(this, &FIFOBufferStreamTest::onReadable);
	iostr.writable -= delegate(this, &FIFOBufferStreamTest::onWritable);
}


void FIFOBufferStreamTest::onReadable(bool& b)
{
	if (b) ++_notToReadable;
	else ++_readableToNot;
};


void FIFOBufferStreamTest::onWritable(bool& b)
{
	if (b) ++_notToWritable;
	else ++_writableToNot;
}


void FIFOBufferStreamTest::setUp()
{
	_readableToNot = 0;
	_notToReadable = 0;
	_writableToNot = 0;
	_notToWritable = 0;
}


void FIFOBufferStreamTest::tearDown()
{
}


CppUnit::Test* FIFOBufferStreamTest::suite()
{
	CppUnit::TestSuite* pSuite = new CppUnit::TestSuite("FIFOBufferStreamTest");

	CppUnit_addTest(pSuite, FIFOBufferStreamTest, testInput);
	CppUnit_addTest(pSuite, FIFOBufferStreamTest, testOutput);
	CppUnit_addTest(pSuite, FIFOBufferStreamTest, testNotify);

	return pSuite;
}
