//
// NumberParserTest.h
//
// $Id: //poco/1.4/Foundation/testsuite/src/NumberParserTest.h#1 $
//
// Definition of the NumberParserTest class.
//
// Copyright (c) 2004-2006, Applied Informatics Software Engineering GmbH.
// and Contributors.
//
// Permission is hereby granted, free of charge, to any person or organization
// obtaining a copy of the software and accompanying documentation covered by
// this license (the "Software") to use, reproduce, display, distribute,
// execute, and transmit the Software, and to prepare derivative works of the
// Software, and to permit third-parties to whom the Software is furnished to
// do so, all subject to the following:
// 
// The copyright notices in the Software and this entire statement, including
// the above license grant, this restriction and the following disclaimer,
// must be included in all copies of the Software, in whole or in part, and
// all derivative works of the Software, unless such copies or derivative
// works are solely in the form of machine-executable object code generated by
// a source language processor.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
// SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
// FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.
//


#ifndef NumberParserTest_INCLUDED
#define NumberParserTest_INCLUDED


#include "Poco/Foundation.h"
#include "CppUnit/TestCase.h"
#include "Poco/NumberParser.h"
#include "Poco/NumberFormatter.h"
#undef max
#undef min
#include <limits>

class NumberParserTest: public CppUnit::TestCase
{
public:
	NumberParserTest(const std::string& name);
	~NumberParserTest();

	void testParse();
	void testLimits();
	void testParseError();

	void setUp();
	void tearDown();

	static CppUnit::Test* suite();

private:

	template <class T> bool testUpperLimit()
	{
		T n = std::numeric_limits<T>::max();
		std::string s = Poco::NumberFormatter::format(n);
		if (std::numeric_limits<T>::is_signed)
			return Poco::NumberParser::parse(s) == n;
		else
			return Poco::NumberParser::parseUnsigned(s) == n;
	}

	template <class T> bool testLowerLimit()
	{
		T n = std::numeric_limits<T>::min();
		std::string s = Poco::NumberFormatter::format(n);
		return Poco::NumberParser::parse(s) == n;
	}

#if defined(POCO_HAVE_INT64)
	template <class T> bool testUpperLimit64()
	{
		T n = std::numeric_limits<T>::max();
		std::string s = Poco::NumberFormatter::format(n);
		if (std::numeric_limits<T>::is_signed)
			return Poco::NumberParser::parse64(s) == n;
		else
			return Poco::NumberParser::parseUnsigned64(s) == n;
	}

	template <class T> bool testLowerLimit64()
	{
		T n = std::numeric_limits<T>::min();
		std::string s = Poco::NumberFormatter::format(n);
		return Poco::NumberParser::parse64(s) == n;
	}
#endif
};


#endif // NumberParserTest_INCLUDED
